@@ -1,9 +1,9 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2020-2021 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2020-2024 -- leonerd@leonerd.org.uk
 
-package XS::Parse::Sublike 0.16;
+package XS::Parse::Sublike 0.32;
 
 use v5.14;
 use warnings;
@@ -32,6 +32,8 @@ changes or at least a rebuild of any module that depends on it.
 
 =head1 XS FUNCTIONS
 
+=for highlighter language=c
+
 =head2 boot_xs_parse_sublike
 
   void boot_xs_parse_sublike(double ver)
@@ -106,6 +108,37 @@ stage functions when they are invoked.
 Note that this function is now vaguely discouraged, in favour of using a
 prefixing keyword instead, by using the C<XS_PARSE_SUBLIKE_FLAG_PREFIX> flag.
 
+=head2 xps_signature_add_param
+
+   void xps_signature_add_param(struct XSParseSublikeContext *ctx,
+      struct XPSSignatureParamDetails *details);
+
+I<Since version 0.31; experimental.>
+
+This B<experimental> function may only be called during the C<start_signature>
+or C<finish_signature> hook stages. It is used to insert extra signature
+parameters, either at the beginning (when called by the start hook), or at the
+end (when called by the finish hook). It takes details of the parameter to add
+from an addressed structure, which has the following fields.
+
+   struct XPSSignatureParamDetails {
+      U32 ver;
+
+      char sigil;
+      PADOFFSET padix;
+   };
+
+The caller must set the I<ver> field equal to C<XSPARSESUBLIKE_ABI_VERSION>.
+
+The I<sigil> field gives the leading sigil of the parameter; C<$> for
+mandatory scalars, C<@> or C<%> for a final slurpy. The I<padix> field gives
+the pad offset for a pad variable to store the value into. The caller is
+(currently) responsible for creating that pad variable.
+
+At the present version, this API cannot create optional, or named parameters.
+These abilities may be added in a later version which expands on the
+structure's definition to add new fields to support this.
+
 =head1 PARSE CONTEXT
 
 The various hook stages all share state about the ongoing parse process using
@@ -157,6 +190,14 @@ its given name. Note that it is not possible to enable this flag without also
 enabling C<XS_PARSE_SUBLIKE_ACTION_SET_CVNAME>. This is set by default if a
 name was found.
 
+=item XS_PARSE_SUBLIKE_ACTION_INSTALL_LEXICAL
+
+If set, the newly-constructed CV will be installed into the currently
+compiling lexical pad as its given name. This is only available on Perl
+version 5.18 or above, and conflicts with the alternative of
+C<XS_PARSE_SUBLIKE_ACTION_INSTALL_SYMBOL>. This is set by default if a name
+was found I<and> the C<my> keyword appeared before the construction.
+
 =item XS_PARSE_SUBLIKE_ACTION_REFGEN_ANONCODE
 
 If set, the syntax will yield the C<OP_REFGEN> / C<OP_ANONCODE> optree
@@ -186,10 +227,6 @@ operation. The following flags are defined.
 
 =over 4
 
-=item XS_PARSE_SUBLIKE_FLAG_FILTERATTRS
-
-If set, the optional C<filter_attr> stage will be invoked.
-
 =item XS_PARSE_SUBLIKE_FLAG_BODY_OPTIONAL
 
 If B<not> set, the I<require_parts> field will imply the
@@ -203,6 +240,27 @@ If set, the keyword is considered to be a prefix that can be placed in front
 of C<sub> or another sub-like keyword, to add its set of hooks in addition to
 those of the following keyword. These prefices may be further stacked.
 
+=item XS_PARSE_SUBLIKE_FLAG_ALLOW_PKGNAME
+
+I<Since version 0.31.>
+
+If B<not> set, then fully-qualified identifiers that include a package name
+are not allowed when declaring a named function. If all hooks agree by all
+setting the flag, then the name may be fully-qualified to add the
+newly-declared function into a different package.
+
+=item XS_PARSE_SUBLIKE_FLAG_SIGNATURE_NAMED_PARAMS
+
+If set, use the extended signature parser of this module when parsing a
+signature and additionally permit the use of named parameter syntax, as
+documented in L<Sublike::Extended>.
+
+=item XS_PARSE_SUBLIKE_FLAG_SIGNATURE_PARAM_ATTRIBUTES
+
+If set, use the extended signature parser of this module when parsing a
+signature and additionally permit the use of attribute declarations on
+parameter variables, as documented in L<Sublike::Extended>.
+
 =back
 
 In addition there are two C<U8> fields named I<require_parts> and
@@ -242,10 +300,16 @@ definition.
 
 The parameter signature of the function.
 
-This part can be skipped, but the bit is ignored when in I<require_parts>. It
-is always permitted not to provide a signature for a function definition,
-because such syntax only applies when C<use feature 'signatures'> is in
-effect, and only on supporting perl versions.
+This part can be skipped, but it is always permitted not to provide a
+signature for a function definition even if the bit it set in
+I<require_parts>. This is because such syntax only applies when
+C<use feature 'signatures'> is in effect, and only on supporting perl
+versions.
+
+However, setting the bit in I<require_parts> instead has the effect of
+enabling C<use feature 'signatures'> (at least on supporting perl versions),
+thus permitting the syntax to use a signature even if the signatures feature
+was not previously enabled.
 
 =item XS_PARSE_SUBLIKE_PART_BODY
 
@@ -303,16 +367,16 @@ Invoked just before C<start_subparse()> is called.
 At this point the optional sub attributes are parsed and filled into the
 C<attrs> field of the context, then C<block_start()> is called.
 
-=head2 The optional C<filter_attr> Stage
+=head2 The C<filter_attr> Stage
 
    bool (*filter_attr)(pTHX_ struct XSParseSublikeContext *ctx,
       SV *attr, SV *val, void *hookdata);
 
-If the I<flags> field includes C<XS_PARSE_SUBLIKE_FLAG_FILTERATTRS> then each
-individual attribute is passed through this optional filter function
-immediately as each is parsed. I<attr> will be a string SV containing the name
-of the attribute, and I<val> will either be C<NULL>, or a string SV containing
-the contents of the parens after its name (without the parens themselves).
+If this optional stage is defined, then each individual attribute is passed
+through this optional filter function immediately as each is parsed. I<attr>
+will be a string SV containing the name of the attribute, and I<val> will
+either be C<NULL>, or a string SV containing the contents of the parens after
+its name (without the parens themselves).
 
 If the filter returns C<true>, it indicates that it has in some way handled
 the attribute and it should not be added to the list given to C<newATTRSUB()>.
@@ -328,12 +392,27 @@ may wish to perform any alterations of C<PL_compcv> or related, inspect or
 alter the lexical pad, provide hints hash values, or any other tasks before
 the signature and code body are parsed.
 
+=head2 Parse Signature
+
+If the perl version supports subroutine signatures, and the feature is enabled
+at this point, then an optional signatured is expected. 
+
+I<Since version 0.31>: if the open parenthesis of a signature declaration is
+found, then the C<start_signature> stage is invoked. Once the signature has
+been parsed, the C<finish_signature> stage is invoked.
+
+   void (*start_signature)(pTHX_ struct XSParseSublikeContext *ctx, void *hookdata);
+
+   void (*finish_signature)(pTHX_ struct XSParseSublikeContext *ctx, void *hookdata);
+
+Code in either of these hook stages is permitted to call
+L</xps_signature_add_param>.
+
 =head2 Parse Body
 
 At this point, the main body of the function is parsed and the optree is
-stored in the C<body> field of the context. If the perl version supports sub
-signatures and they are enabled and found, the body will be prefixed with the
-signature ops as well.
+stored in the C<body> field of the context. If a subroutine signature was
+found, the body will be prefixed with the signature ops as well.
 
 =head2 The C<pre_blockend> Stage
 
